import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:flutter_slick/utils/action_dialog_options.dart';
import 'package:provider/provider.dart';

class StandardDialog extends StatefulWidget {
  final Widget? content;
  final Widget? title;
  final Future<bool> Function()? beforeClose;
  final List<Widget> actions;

  const StandardDialog({
    super.key,
    required this.title,
    required this.content,
    required this.actions,
    this.beforeClose,
  });

  @override
  State<StandardDialog> createState() => _StandardDialog();
}

class _StandardDialog extends State<StandardDialog> {
  @override
  PopScope build(BuildContext context) {
    return PopScope(
      canPop: false,
      onPopInvoked: (didPop) async {
        if (didPop) {
          Provider.of<DirtyStateMonitor>(context, listen: false).clearDirty();
          return;
        }
        if (widget.beforeClose == null || (await widget.beforeClose!())) {
          // ignore: use_build_context_synchronously
          Navigator.of(context).pop(ActionDialogOptions.cancel);
        }
      },
      child: AlertDialog(
        title: widget.title,
        content: widget.content,
        actions: widget.actions,
      ),
    );
  }

  @override
  void initState() {
    if (kDebugMode &&
        Provider.of<DirtyStateMonitor>(context, listen: false).isDirty) {
      // ignore: avoid_print
      print(
          "Warning: IsDirty is set to True and will be reset after closing the dialog. The framework does not currently support multiple dirty states");
    }
    super.initState();
  }
}
