import 'package:flutter/material.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:provider/provider.dart';

import '../text/body.dart';

class CheckboxField extends StatefulWidget {
  final bool autofocus;
  final String labelText;
  final bool initialValue;
  final void Function(bool) onChanged;

  const CheckboxField({
    super.key,
    this.autofocus = false,
    required this.labelText,
    required this.initialValue,
    required this.onChanged,
  });

  @override
  State<CheckboxField> createState() => _CheckboxFieldState();
}

class _CheckboxFieldState extends State<CheckboxField> {
  bool _isChecked = false;

  @override
  void initState() {
    super.initState();
    _isChecked = widget.initialValue;
  }

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: () {
        setState(() => _isChecked = !_isChecked);
        widget.onChanged(_isChecked);
      },
      child: Row(
        children: [
          Checkbox(
            autofocus: widget.autofocus,
            value: _isChecked,
            onChanged: (v) {
              setState(() => _isChecked = !_isChecked);
              Provider.of<DirtyStateMonitor>(context, listen: false).setDirty();
              widget.onChanged(_isChecked);
            },
          ),
          Expanded(
            child: BodyText(widget.labelText),
          ),
        ],
      ),
    );
  }
}
