// @dart = 2.17

import 'package:flutter/material.dart';
import 'package:dropdown_search/dropdown_search.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:provider/provider.dart';

class DropdownField<T> extends StatelessWidget {
  final List<T> items;
  final T? selectedItem;
  final String Function(T) itemAsString;
  final String labelText;
  final Function(T?) onChanged;
  final String? Function(T?)? validator;

  const DropdownField({
    Key? key,
    required this.items,
    required this.selectedItem,
    required this.itemAsString,
    required this.labelText,
    required this.onChanged,
    this.validator,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return DropdownSearch<T>(
      items: items,
      selectedItem: selectedItem,
      itemAsString: itemAsString,
      dropdownDecoratorProps: DropDownDecoratorProps(
        dropdownSearchDecoration: InputDecoration(
          labelText: labelText,
        ),
        baseStyle: Theme.of(context).textTheme.bodyLarge,
      ),
      onChanged: (value) {
        Provider.of<DirtyStateMonitor>(context, listen: false).setDirty();
        onChanged(value);
      },
      validator: validator,
    );
  }
}
