import 'package:flutter/material.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:flutter_slick/utils/validators.dart';
import 'package:provider/provider.dart';

class EmailField extends StatefulWidget {
  final String? initialValue;
  final FieldValidator<String>? validator;
  final bool? enabled;
  final String labelText;
  final FocusNode? focusNode;
  final TextEditingController? controller;
  final bool autofocus;
  final void Function(String?) onChanged;
  final ValueChanged<String>? onFieldSubmitted;

  const EmailField({
    super.key,
    this.initialValue,
    this.validator,
    this.enabled,
    this.onFieldSubmitted,
    this.focusNode,
    this.controller,
    this.autofocus = false,
    required this.labelText,
    required this.onChanged,
  });

  @override
  State<EmailField> createState() => _EmailFieldState();
}

class _EmailFieldState extends State<EmailField> {
  String? _currentValue;
  late FocusNode focusNode;

  @override
  void initState() {
    super.initState();

    _currentValue = widget.initialValue;
    focusNode = widget.focusNode ?? FocusNode();
    focusNode.addListener(() {
      if (focusNode.hasFocus) return;
      if (_currentValue != widget.initialValue) {
        Provider.of<DirtyStateMonitor>(context, listen: false).setDirty();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    var validator = widget.validator;
    return TextFormField(
      enabled: widget.enabled,
      initialValue: widget.initialValue,
      controller: widget.controller,
      onChanged: (value) {
        _currentValue = value;
        widget.onChanged(value);
      },
      validator: CombinedFieldValidator<String>([
        EmailFieldValidator(),
        if (validator != null) validator,
      ]),
      autofocus: widget.autofocus,
      keyboardType: TextInputType.emailAddress,
      decoration: InputDecoration(
        labelText: widget.labelText,
      ),
      onFieldSubmitted: widget.onFieldSubmitted,
      autofillHints: const [
        AutofillHints.username,
        AutofillHints.email,
      ],
      focusNode: focusNode,
    );
  }
}
