import 'package:flutter/material.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:provider/provider.dart';

class PasswordField extends StatefulWidget {
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final ValueChanged<String>? onFieldSubmitted;
  final TextInputAction? textInputAction;
  final Function()? onEditingComplete;
  final TextEditingController? controller;
  final bool enabled;
  final bool autofocus;
  final String labelText;
  final Function(String?) onChanged;
  final TextInputType? keyboardType;
  final GestureTapCallback? onTap;

  const PasswordField({
    super.key,
    this.initialValue,
    this.validator,
    this.onFieldSubmitted,
    this.textInputAction,
    this.onEditingComplete,
    this.controller,
    this.enabled = true,
    this.autofocus = false,
    required this.labelText,
    required this.onChanged,
    this.keyboardType = TextInputType.text,
    this.onTap,
  });

  @override
  State<PasswordField> createState() => _PasswordFieldState();
}

class _PasswordFieldState extends State<PasswordField> {
  bool _isHidden = true;
  String? _currentValue;
  late FocusNode focusNode;

  @override
  void initState() {
    super.initState();
    _currentValue = widget.initialValue;
    focusNode = FocusNode();
    focusNode.addListener(() {
      if (focusNode.hasFocus) return;
      if (_currentValue != widget.initialValue) {
        Provider.of<DirtyStateMonitor>(context, listen: false).setDirty();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    var theme = Theme.of(context);
    return TextFormField(
      initialValue: widget.initialValue,
      controller: widget.controller,
      onChanged: (value) {
        _currentValue = value;
        widget.onChanged(value);
      },
      validator: widget.validator,
      obscureText: _isHidden,
      maxLines: 1,
      keyboardType: widget.keyboardType,
      focusNode: focusNode,
      decoration: InputDecoration(
        enabled: widget.enabled,
        labelText: widget.labelText,
        suffixIcon: InkWell(
          focusNode: FocusNode(skipTraversal: true),
          onTap: _toggleHidden,
          child: Icon(
            Icons.visibility,
            color: _isHidden ? theme.disabledColor : null,
          ),
        ),
      ),
      onFieldSubmitted: widget.onFieldSubmitted,
      autofillHints: const [AutofillHints.password],
      autofocus: widget.autofocus,
      textInputAction: widget.textInputAction,
      onEditingComplete: widget.onEditingComplete,
      onTap: widget.onTap,
    );
  }

  void _toggleHidden() {
    setState(() => _isHidden = !_isHidden);
  }
}
