import 'package:flutter/material.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:provider/provider.dart';

class StandardTextField extends StatefulWidget {
  final String? initialValue;
  final FormFieldValidator<String>? validator;
  final int? minLines;
  final int? maxLines;
  final int? maxLength;
  final TextInputAction? textInputAction;
  final FocusNode? focusNode;
  final Function()? onEditingComplete;
  final Function()? onFocusLost;
  final TextEditingController? controller;
  final bool enabled;
  final bool autofocus;
  final String? labelText;
  final Function(String?) onChanged;
  final GestureTapCallback? onTap;
  final Iterable<String>? autofillHints;
  final InputDecoration? decoration;
  final TextStyle? style;
  final TextInputType keyboardType;

  const StandardTextField({
    super.key,
    this.initialValue,
    this.validator,
    this.minLines,
    this.maxLines,
    this.maxLength,
    this.focusNode,
    this.textInputAction,
    this.onEditingComplete,
    this.controller,
    this.enabled = true,
    this.autofocus = false,
    this.labelText,
    required this.onChanged,
    this.onTap,
    this.autofillHints = const [],
    this.decoration,
    this.style,
    this.onFocusLost,
    this.keyboardType = TextInputType.text,
  });

  @override
  State<StandardTextField> createState() => _StandardTextFieldState();
}

class _StandardTextFieldState extends State<StandardTextField> {
  String? _currentValue;
  late FocusNode focusNode;

  @override
  void initState() {
    super.initState();

    _currentValue = widget.initialValue;
    focusNode = widget.focusNode ?? FocusNode();
    focusNode.addListener(() {
      if (focusNode.hasFocus) return;
      if (_currentValue != widget.initialValue) {
        Provider.of<DirtyStateMonitor>(context, listen: false).setDirty();
      }
      if (widget.onFocusLost != null) {
        widget.onFocusLost!();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    InputDecoration decoration;
    if (widget.decoration != null) {
      decoration = widget.decoration!.copyWith(
        labelText: widget.labelText,
        enabled: widget.enabled,
      );
    } else {
      decoration = InputDecoration(
        labelText: widget.labelText,
        enabled: widget.enabled,
      );
    }

    return TextFormField(
      initialValue: widget.initialValue,
      onChanged: (value) {
        _currentValue = value;
        widget.onChanged(value);
      },
      style: widget.style,
      validator: widget.validator,
      keyboardType: widget.keyboardType,
      decoration: decoration,
      minLines: widget.minLines,
      maxLines: widget.maxLines,
      maxLength: widget.maxLength,
      autofocus: widget.autofocus,
      focusNode: focusNode,
      textInputAction: widget.textInputAction,
      onEditingComplete: widget.onEditingComplete,
      controller: widget.controller,
      onTap: widget.onTap,
      autofillHints: widget.autofillHints,
    );
  }
}
