import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:flutter_slick/services/dirty_state_monitor.dart';
import 'package:provider/provider.dart';

import 'route_path.dart';
import 'paths_builder.dart';
import 'router_delegate.dart';
import 'route_information_parser.dart';

class NavigationLogic extends ChangeNotifier {
  final BuildContext context;
  final Widget Function() initialScreen;

  final PathsBuilder rootPaths;
  final Stream? authState;

  late StandardRouterDelegate routerDelegate;
  late StandardRouteInformationParser routerInformationParser;

  late Widget _currentScreen;
  late RoutePath _routePath;

  RoutePath? _rewrittenRoutePath;

  StreamSubscription? _onAuthStateChangedSubscription;

  RoutePath? get routePath => _routePath;

  bool _isLoading = true;
  bool _isInitialising = true;

  bool get isLoading => _isLoading;

  set isLoading(bool value) {
    _isLoading = value;
    notifyListeners();
  }

  Widget get currentScreen => _currentScreen;

  NavigationLogic({
    required this.context,
    required this.initialScreen,
    required this.rootPaths,
    this.authState,
  }) : _currentScreen = initialScreen() {
    _routePath = rootPaths.bareBuilder('', initialScreen)();
    routerDelegate = StandardRouterDelegate(context, this);
    routerInformationParser = StandardRouteInformationParser(rootPaths);
    final authState = this.authState;
    if (authState != null) {
      _onAuthStateChangedSubscription = authState.listen(_onAuthStateChanged);
    } else {
      _isInitialising = false;
    }
  }

  Future<void> setRoutePath(RoutePath routePath) async {
    Provider.of<DirtyStateMonitor>(context, listen: false).clearDirty();
    if (_isInitialising) {
      _routePath = routePath;
      _currentScreen = initialScreen();
    } else {
      isLoading = true;
      // Detemine if a redirect is indicated
      RoutePath? currentPath = routePath;
      _rewrittenRoutePath = null;
      while (currentPath != null) {
        var result = await currentPath.rewrite(context);
        if (result == null) {
          _routePath = currentPath;
        } else {
          _rewrittenRoutePath = result;
        }
        currentPath = result;
      }
      // ignore: use_build_context_synchronously
      _currentScreen = await _routePath.build(context);
      // TO-DO: Reimplement
      // Attempt to build the current route path into a screen
      // try {
      // _currentScreen = await _routePath.build(context);
      // } on ApiException catch (e) {
      //   if (e.code == 404) {
      //     _currentScreen = rootPaths.notFoundScreen();
      //   } else {
      //     rethrow;
      //   }
      // }
      isLoading = false;
    }
  }

  Future<void> _onAuthStateChanged(_) async {
    _isInitialising = false;
    await setRoutePath(_rewrittenRoutePath ?? _routePath);
  }

  @override
  void dispose() {
    _onAuthStateChangedSubscription?.cancel();
    super.dispose();
  }
}
