import 'package:flutter/material.dart';

import 'logic.dart';
import 'route_path.dart';

class StandardRouterDelegate<User> extends RouterDelegate<RoutePath>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<RoutePath> {
  final BuildContext context;

  final NavigationLogic _navigationLogic;

  final List<RoutePath> _history = [];

  @override
  final navigatorKey = GlobalKey<NavigatorState>();

  @override
  get currentConfiguration => _navigationLogic.routePath;

  StandardRouterDelegate(this.context, this._navigationLogic) {
    _navigationLogic.addListener(notifyListeners);
  }

  @override
  Future<bool> popRoute() async {
    if (_history.isEmpty) {
      return false;
    } else {
      await setNewRoutePath(_history.last);
      return true;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navigatorKey,
      onPopPage: (route, result) => route.didPop(result),
      pages: [
        MaterialPage(
          child: _navigationLogic.currentScreen,
        ),
      ],
    );
  }

  @override
  Future<void> setNewRoutePath(RoutePath configuration) async {
    if (_history.isNotEmpty && _history.last == configuration) {
      _history.removeLast();
    } else {
      _history.add(configuration);
    }
    await _navigationLogic.setRoutePath(configuration);
  }
}
