import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

import 'authentication.dart';

abstract class ApiService<User, UserModel> {
  final BuildContext context;

  final AuthenticationService<User> _authenticationService;

  final _authStateController = StreamController<UserModel?>();

  late StreamSubscription<User?> _onAuthStateChangeSubscription;
  late StreamSubscription<String?> _onTokenChangeSubscription;

  UserModel? _user;

  Stream<UserModel?> get authState => _authStateController.stream;
  UserModel? get user => _user;

  ApiService({
    required this.context,
  }) : _authenticationService = Provider.of(context, listen: false) {
    _onAuthStateChangeSubscription =
        _authenticationService.authState.listen(_onAuthStateChanged);
    _onTokenChangeSubscription =
        _authenticationService.tokenState.listen(_onTokenChanged);
  }

  Future<UserModel> getUser();

  void setToken(String? token);

  void _onAuthStateChanged(User? user) async {
    if (user == null) {
      _user = null;
    } else {
      _user = await getUser();
    }
    _authStateController.add(_user);
  }

  void _onTokenChanged(String? token) async {
    setToken(token);
  }

  void dispose() {
    _onAuthStateChangeSubscription.cancel();
    _onTokenChangeSubscription.cancel();
  }
}
