// @dart = 2.17

import 'dart:async';

class AuthenticationException implements Exception {
  final String message;

  AuthenticationException(this.message);
}

abstract class AuthenticationService<User> {
  final authStateChangeController = StreamController<User?>.broadcast();
  final tokenChangeController = StreamController<String?>.broadcast();

  Stream<User?> get authState => authStateChangeController.stream;
  Stream<String?> get tokenState => tokenChangeController.stream;

  User? get user;

  Future<void> refreshToken(User user);

  Future<void> signIn(String email, String password);

  Future<void> signOut();

  Future<void> createUser(String email, String password);

  Future<void> resetPasswordReset(String email);

  Future<void> confirmPasswordReset(String code, String newPassword);

  Future<void> sendResetPasswordEmail(String? email);

  Future<void> dispose() async {
    await authStateChangeController.close();
    await tokenChangeController.close();
  }
}
